@@ -105,7 +105,7 @@ struct grub_uhci_qh
   /* Queue heads are aligned on 16 bytes, pad so a queue head is 16
      bytes so we can store many in a 4K page.  */
   grub_uint8_t pad[8];
-} __attribute__ ((packed));
+} GRUB_PACKED;
 
 /* UHCI Transfer Descriptor.  */
 struct grub_uhci_td
@@ -129,7 +129,7 @@ struct grub_uhci_td
 
   /* 3 additional 32 bits words reserved for the Host Controller Driver.  */
   grub_uint32_t data[3];
-} __attribute__ ((packed));
+} GRUB_PACKED;
 
 typedef volatile struct grub_uhci_td *grub_uhci_td_t;
 typedef volatile struct grub_uhci_qh *grub_uhci_qh_t;
@@ -189,11 +189,6 @@ grub_uhci_writereg32 (struct grub_uhci *u,
   grub_outl (val, u->iobase + reg);
 }
 
-static grub_err_t
-grub_uhci_portstatus (grub_usb_controller_t dev,
-		      unsigned int port, unsigned int enable);
-
-
 /* Iterate over all PCI devices.  Determine if a device is an UHCI
    controller.  If this is the case, initialize it.  */
 static int
@@ -222,12 +217,6 @@ grub_uhci_pci_iter (grub_pci_device_t dev,
   if (class != 0x0c || subclass != 0x03 || interf != 0x00)
     return 0;
 
-  /* Set bus master - needed for coreboot or broken BIOSes */
-  addr = grub_pci_make_address (dev, GRUB_PCI_REG_COMMAND);
-  grub_pci_write_word(addr, GRUB_PCI_COMMAND_IO_ENABLED
-		      | GRUB_PCI_COMMAND_BUS_MASTER
-		      | grub_pci_read_word (addr));
-
   /* Determine IO base address.  */
   addr = grub_pci_make_address (dev, GRUB_PCI_REG_ADDRESS_REG4);
   base = grub_pci_read (addr);
@@ -238,6 +227,13 @@ grub_uhci_pci_iter (grub_pci_device_t dev,
   if ((base & GRUB_UHCI_IOMASK) == 0)
     return 0;
 
+  /* Set bus master - needed for coreboot or broken BIOSes */
+  addr = grub_pci_make_address (dev, GRUB_PCI_REG_COMMAND);
+  grub_pci_write_word(addr, GRUB_PCI_COMMAND_IO_ENABLED
+		      | GRUB_PCI_COMMAND_BUS_MASTER
+		      | GRUB_PCI_COMMAND_MEM_ENABLED
+		      | grub_pci_read_word (addr));
+
   grub_dprintf ("uhci", "base = %x\n", base);
 
   /* Allocate memory for the controller and register it.  */
@@ -629,9 +625,7 @@ grub_uhci_check_transfer (grub_usb_controller_t dev,
       return GRUB_USB_ERR_NONE;
     }
 
-  grub_dprintf ("uhci", "t status=0x%02x\n", errtd->ctrl_status);
-
-  if (!(errtd->ctrl_status & (1 << 23)))
+  if (errtd && !(errtd->ctrl_status & (1 << 23)))
     {
       grub_usb_err_t err = GRUB_USB_ERR_NONE;
 
@@ -715,7 +709,7 @@ grub_uhci_iterate (grub_usb_controller_iterate_hook_t hook, void *hook_data)
   return 0;
 }
 
-static grub_err_t
+static grub_usb_err_t
 grub_uhci_portstatus (grub_usb_controller_t dev,
 		      unsigned int port, unsigned int enable)
 {
@@ -733,8 +727,7 @@ grub_uhci_portstatus (grub_usb_controller_t dev,
   else if (port == 1)
     reg = GRUB_UHCI_REG_PORTSC2;
   else
-    return grub_error (GRUB_ERR_OUT_OF_RANGE,
-		       "UHCI Root Hub port does not exist");
+    return GRUB_USB_ERR_INTERNAL;
 
   status = grub_uhci_readreg16 (u, reg);
   grub_dprintf ("uhci", "detect=0x%02x\n", status);
@@ -747,11 +740,11 @@ grub_uhci_portstatus (grub_usb_controller_t dev,
       endtime = grub_get_time_ms () + 1000;
       while ((grub_uhci_readreg16 (u, reg) & (1 << 2)))
         if (grub_get_time_ms () > endtime)
-          return grub_error (GRUB_ERR_IO, "UHCI Timed out - disable");
+          return GRUB_USB_ERR_TIMEOUT;
 
       status = grub_uhci_readreg16 (u, reg);
       grub_dprintf ("uhci", ">3detect=0x%02x\n", status);
-      return GRUB_ERR_NONE;
+      return GRUB_USB_ERR_NONE;
     }
     
   /* Reset the port.  */
@@ -782,7 +775,7 @@ grub_uhci_portstatus (grub_usb_controller_t dev,
   endtime = grub_get_time_ms () + 1000;
   while (! ((status = grub_uhci_readreg16 (u, reg)) & (1 << 2)))
     if (grub_get_time_ms () > endtime)
-      return grub_error (GRUB_ERR_IO, "UHCI Timed out - enable");
+      return GRUB_USB_ERR_TIMEOUT;
 
   /* Reset recovery time */
   grub_millisleep (10);
@@ -792,7 +785,7 @@ grub_uhci_portstatus (grub_usb_controller_t dev,
   grub_dprintf ("uhci", ">3detect=0x%02x\n", status);
 
 
-  return GRUB_ERR_NONE;
+  return GRUB_USB_ERR_NONE;
 }
 
 static grub_usb_speed_t
